"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CacheLfuInterceptor = void 0;
const common_1 = require("@nestjs/common");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const cache_module_1 = require("../cache.module");
const inject_cache_decorator_1 = require("../inject-cache.decorator");
let CacheLfuInterceptor = class CacheLfuInterceptor {
    constructor(cacheService) {
        this.cacheService = cacheService;
        this.cacheModuleOptions = cache_module_1.CacheModule.getInstance();
    }
    async intercept(context, next) {
        const host = context.switchToHttp();
        const request = host.getRequest();
        const { method, url } = request;
        if (method !== 'GET')
            return next.handle();
        const { ttl, max, resetExpires = false, hashKeyName = `LFU`, sortedSetKeyName = `LfuPriority`, } = this.cacheModuleOptions.LFU;
        const key = `${hashKeyName}:${url}`;
        const cachedResult = await this.cacheService.hGet(hashKeyName, key);
        if (cachedResult) {
            resetExpires &&
                this.cacheService.expire(hashKeyName, ttl) &&
                this.cacheService.expire(sortedSetKeyName, ttl);
            this.cacheService.zIncrBy(sortedSetKeyName, 1, key);
            return (0, rxjs_1.of)(JSON.parse(cachedResult));
        }
        else {
            return next.handle().pipe((0, operators_1.tap)(async (data) => {
                const length = await this.cacheService.zCard(sortedSetKeyName);
                if (length >= max)
                    this.cacheService.hDel(hashKeyName, (await this.cacheService.zPopMin(sortedSetKeyName)).value);
                this.cacheService.zIncrBy(sortedSetKeyName, 1, key);
                this.cacheService.hSet(hashKeyName, key, JSON.stringify(data));
                (resetExpires || length == 0) &&
                    this.cacheService.expire(hashKeyName, ttl) &&
                    this.cacheService.expire(sortedSetKeyName, ttl);
            }));
        }
    }
};
exports.CacheLfuInterceptor = CacheLfuInterceptor;
exports.CacheLfuInterceptor = CacheLfuInterceptor = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, inject_cache_decorator_1.InjectCache)()),
    __metadata("design:paramtypes", [Object])
], CacheLfuInterceptor);
//# sourceMappingURL=cache-lfu.interceptor.js.map